#include "misc/PCITypes.h"

static const char* unclassified_device(PCI::DeviceType type)
{
    switch (type.dev_subclass)
    {
    case 0x0: return "Non-VGA-Compatible Unclassified Device";
    case 0x1: return "VGA-Compatible Unclassified Device";

    default: return "Unclassified";
    }
}

static const char* display_controller(PCI::DeviceType type)
{
    switch (type.dev_subclass)
    {
    case 0x0: return type.prog_if == 1 ? "8514-Compatible Controller" : "VGA Controller";
    case 0x1: return "XGA Controller";
    case 0x2: return "3D Controller (Not VGA-Compatible)";
    case 0x80: return "Display Controller";

    default: return "Unknown Display Controller";
    }
}

static const char* memory_controller(PCI::DeviceType type)
{
    switch (type.dev_subclass)
    {
    case 0x0: return "RAM Controller";
    case 0x1: return "Flash Controller";
    case 0x80: return "Memory Controller";

    default: return "Unknown Memory Controller";
    }
}

static const char* processor(PCI::DeviceType type)
{
    switch (type.dev_subclass)
    {
    case 0x0: return "Processor (386)";
    case 0x1: return "Processor (486)";
    case 0x2: return "Processor (Pentium)";
    case 0x3: return "Processor (Pentium Pro)";
    case 0x10: return "Processor (Alpha)";
    case 0x20: return "Processor (PowerPC)";
    case 0x30: return "Processor (MIPS)";
    case 0x40: return "Co-Processor";
    case 0x80: return "Processor";

    default: return "Unknown Processor";
    }
}

const char* pci_type_name(PCI::DeviceType type)
{
    switch (type.dev_class)
    {
    case 0x0: return unclassified_device(type);
    case 0x1: return "Mass Storage Controller";
    case 0x2: return "Network Controller";
    case 0x3: return display_controller(type);
    case 0x4: return "Multimedia Controller";
    case 0x5: return memory_controller(type);
    case 0x6: return "Bridge";
    case 0x7: return "Simple Communication Controller";
    case 0x8: return "Base System Peripheral";
    case 0x9: return "Input Device Controller";
    case 0xA: return "Docking Station";
    case 0xB: return processor(type);
    case 0xC: return "Serial Bus Controller";
    case 0xD: return "Wireless Controller";
    case 0xE: return "Intelligent Controller (I20)";
    case 0xF: return "Satellite Communication Controller";
    case 0x10: return "Encryption Controller";
    case 0x11: return "Signal Processing Controller";
    case 0x12: return "Processing Accelerator";
    case 0x13: return "Non-Essential Instrumentation";
    case 0x40: return "Co-Processor";

    default: return "Unknown";
    }
}